/*******************************************************************************
* File Name: NMIgen.c
* Version 3.30
*
* Description:
*  The PWM User Module consist of an 8 or 16-bit counter with two 8 or 16-bit
*  comparitors. Each instance of this user module is capable of generating
*  two PWM outputs with the same period. The pulse width is selectable between
*  1 and 255/65535. The period is selectable between 2 and 255/65536 clocks.
*  The compare value output may be configured to be active when the present
*  counter is less than or less than/equal to the compare value.
*  A terminal count output is also provided. It generates a pulse one clock
*  width wide when the counter is equal to zero.
*
* Note:
*
*******************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "NMIgen.h"

/* Error message for removed <resource> through optimization */
#ifdef NMIgen_PWMUDB_genblk1_ctrlreg__REMOVED
    #error PWM_v3_30 detected with a constant 0 for the enable or \
         constant 1 for reset. This will prevent the component from operating.
#endif /* NMIgen_PWMUDB_genblk1_ctrlreg__REMOVED */

uint8 NMIgen_initVar = 0u;


/*******************************************************************************
* Function Name: NMIgen_Start
********************************************************************************
*
* Summary:
*  The start function initializes the pwm with the default values, the
*  enables the counter to begin counting.  It does not enable interrupts,
*  the EnableInt command should be called if interrupt generation is required.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  NMIgen_initVar: Is modified when this function is called for the
*   first time. Is used to ensure that initialization happens only once.
*
*******************************************************************************/
void NMIgen_Start(void) 
{
    /* If not Initialized then initialize all required hardware and software */
    if(NMIgen_initVar == 0u)
    {
        NMIgen_Init();
        NMIgen_initVar = 1u;
    }
    NMIgen_Enable();

}


/*******************************************************************************
* Function Name: NMIgen_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the
*  customizer of the component placed onto schematic. Usually called in
*  NMIgen_Start().
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void NMIgen_Init(void) 
{
    #if (NMIgen_UsingFixedFunction || NMIgen_UseControl)
        uint8 ctrl;
    #endif /* (NMIgen_UsingFixedFunction || NMIgen_UseControl) */

    #if(!NMIgen_UsingFixedFunction)
        #if(NMIgen_UseStatus)
            /* Interrupt State Backup for Critical Region*/
            uint8 NMIgen_interruptState;
        #endif /* (NMIgen_UseStatus) */
    #endif /* (!NMIgen_UsingFixedFunction) */

    #if (NMIgen_UsingFixedFunction)
        /* You are allowed to write the compare value (FF only) */
        NMIgen_CONTROL |= NMIgen_CFG0_MODE;
        #if (NMIgen_DeadBand2_4)
            NMIgen_CONTROL |= NMIgen_CFG0_DB;
        #endif /* (NMIgen_DeadBand2_4) */

        ctrl = NMIgen_CONTROL3 & ((uint8 )(~NMIgen_CTRL_CMPMODE1_MASK));
        NMIgen_CONTROL3 = ctrl | NMIgen_DEFAULT_COMPARE1_MODE;

         /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
        NMIgen_RT1 &= ((uint8)(~NMIgen_RT1_MASK));
        NMIgen_RT1 |= NMIgen_SYNC;

        /*Enable DSI Sync all all inputs of the PWM*/
        NMIgen_RT1 &= ((uint8)(~NMIgen_SYNCDSI_MASK));
        NMIgen_RT1 |= NMIgen_SYNCDSI_EN;

    #elif (NMIgen_UseControl)
        /* Set the default compare mode defined in the parameter */
        ctrl = NMIgen_CONTROL & ((uint8)(~NMIgen_CTRL_CMPMODE2_MASK)) &
                ((uint8)(~NMIgen_CTRL_CMPMODE1_MASK));
        NMIgen_CONTROL = ctrl | NMIgen_DEFAULT_COMPARE2_MODE |
                                   NMIgen_DEFAULT_COMPARE1_MODE;
    #endif /* (NMIgen_UsingFixedFunction) */

    #if (!NMIgen_UsingFixedFunction)
        #if (NMIgen_Resolution == 8)
            /* Set FIFO 0 to 1 byte register for period*/
            NMIgen_AUX_CONTROLDP0 |= (NMIgen_AUX_CTRL_FIFO0_CLR);
        #else /* (NMIgen_Resolution == 16)*/
            /* Set FIFO 0 to 1 byte register for period */
            NMIgen_AUX_CONTROLDP0 |= (NMIgen_AUX_CTRL_FIFO0_CLR);
            NMIgen_AUX_CONTROLDP1 |= (NMIgen_AUX_CTRL_FIFO0_CLR);
        #endif /* (NMIgen_Resolution == 8) */

        NMIgen_WriteCounter(NMIgen_INIT_PERIOD_VALUE);
    #endif /* (!NMIgen_UsingFixedFunction) */

    NMIgen_WritePeriod(NMIgen_INIT_PERIOD_VALUE);

        #if (NMIgen_UseOneCompareMode)
            NMIgen_WriteCompare(NMIgen_INIT_COMPARE_VALUE1);
        #else
            NMIgen_WriteCompare1(NMIgen_INIT_COMPARE_VALUE1);
            NMIgen_WriteCompare2(NMIgen_INIT_COMPARE_VALUE2);
        #endif /* (NMIgen_UseOneCompareMode) */

        #if (NMIgen_KillModeMinTime)
            NMIgen_WriteKillTime(NMIgen_MinimumKillTime);
        #endif /* (NMIgen_KillModeMinTime) */

        #if (NMIgen_DeadBandUsed)
            NMIgen_WriteDeadTime(NMIgen_INIT_DEAD_TIME);
        #endif /* (NMIgen_DeadBandUsed) */

    #if (NMIgen_UseStatus || NMIgen_UsingFixedFunction)
        NMIgen_SetInterruptMode(NMIgen_INIT_INTERRUPTS_MODE);
    #endif /* (NMIgen_UseStatus || NMIgen_UsingFixedFunction) */

    #if (NMIgen_UsingFixedFunction)
        /* Globally Enable the Fixed Function Block chosen */
        NMIgen_GLOBAL_ENABLE |= NMIgen_BLOCK_EN_MASK;
        /* Set the Interrupt source to come from the status register */
        NMIgen_CONTROL2 |= NMIgen_CTRL2_IRQ_SEL;
    #else
        #if(NMIgen_UseStatus)

            /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
            /* Enter Critical Region*/
            NMIgen_interruptState = CyEnterCriticalSection();
            /* Use the interrupt output of the status register for IRQ output */
            NMIgen_STATUS_AUX_CTRL |= NMIgen_STATUS_ACTL_INT_EN_MASK;

             /* Exit Critical Region*/
            CyExitCriticalSection(NMIgen_interruptState);

            /* Clear the FIFO to enable the NMIgen_STATUS_FIFOFULL
                   bit to be set on FIFO full. */
            NMIgen_ClearFIFO();
        #endif /* (NMIgen_UseStatus) */
    #endif /* (NMIgen_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: NMIgen_Enable
********************************************************************************
*
* Summary:
*  Enables the PWM block operation
*
* Parameters:
*  None
*
* Return:
*  None
*
* Side Effects:
*  This works only if software enable mode is chosen
*
*******************************************************************************/
void NMIgen_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (NMIgen_UsingFixedFunction)
        NMIgen_GLOBAL_ENABLE |= NMIgen_BLOCK_EN_MASK;
        NMIgen_GLOBAL_STBY_ENABLE |= NMIgen_BLOCK_STBY_EN_MASK;
    #endif /* (NMIgen_UsingFixedFunction) */

    /* Enable the PWM from the control register  */
    #if (NMIgen_UseControl || NMIgen_UsingFixedFunction)
        NMIgen_CONTROL |= NMIgen_CTRL_ENABLE;
    #endif /* (NMIgen_UseControl || NMIgen_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: NMIgen_Stop
********************************************************************************
*
* Summary:
*  The stop function halts the PWM, but does not change any modes or disable
*  interrupts.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Side Effects:
*  If the Enable mode is set to Hardware only then this function
*  has no effect on the operation of the PWM
*
*******************************************************************************/
void NMIgen_Stop(void) 
{
    #if (NMIgen_UseControl || NMIgen_UsingFixedFunction)
        NMIgen_CONTROL &= ((uint8)(~NMIgen_CTRL_ENABLE));
    #endif /* (NMIgen_UseControl || NMIgen_UsingFixedFunction) */

    /* Globally disable the Fixed Function Block chosen */
    #if (NMIgen_UsingFixedFunction)
        NMIgen_GLOBAL_ENABLE &= ((uint8)(~NMIgen_BLOCK_EN_MASK));
        NMIgen_GLOBAL_STBY_ENABLE &= ((uint8)(~NMIgen_BLOCK_STBY_EN_MASK));
    #endif /* (NMIgen_UsingFixedFunction) */
}

#if (NMIgen_UseOneCompareMode)
    #if (NMIgen_CompareMode1SW)


        /*******************************************************************************
        * Function Name: NMIgen_SetCompareMode
        ********************************************************************************
        *
        * Summary:
        *  This function writes the Compare Mode for the pwm output when in Dither mode,
        *  Center Align Mode or One Output Mode.
        *
        * Parameters:
        *  comparemode:  The new compare mode for the PWM output. Use the compare types
        *                defined in the H file as input arguments.
        *
        * Return:
        *  None
        *
        *******************************************************************************/
        void NMIgen_SetCompareMode(uint8 comparemode) 
        {
            #if(NMIgen_UsingFixedFunction)

                #if(0 != NMIgen_CTRL_CMPMODE1_SHIFT)
                    uint8 comparemodemasked = ((uint8)((uint8)comparemode << NMIgen_CTRL_CMPMODE1_SHIFT));
                #else
                    uint8 comparemodemasked = comparemode;
                #endif /* (0 != NMIgen_CTRL_CMPMODE1_SHIFT) */

                NMIgen_CONTROL3 &= ((uint8)(~NMIgen_CTRL_CMPMODE1_MASK)); /*Clear Existing Data */
                NMIgen_CONTROL3 |= comparemodemasked;

            #elif (NMIgen_UseControl)

                #if(0 != NMIgen_CTRL_CMPMODE1_SHIFT)
                    uint8 comparemode1masked = ((uint8)((uint8)comparemode << NMIgen_CTRL_CMPMODE1_SHIFT)) &
                                                NMIgen_CTRL_CMPMODE1_MASK;
                #else
                    uint8 comparemode1masked = comparemode & NMIgen_CTRL_CMPMODE1_MASK;
                #endif /* (0 != NMIgen_CTRL_CMPMODE1_SHIFT) */

                #if(0 != NMIgen_CTRL_CMPMODE2_SHIFT)
                    uint8 comparemode2masked = ((uint8)((uint8)comparemode << NMIgen_CTRL_CMPMODE2_SHIFT)) &
                                               NMIgen_CTRL_CMPMODE2_MASK;
                #else
                    uint8 comparemode2masked = comparemode & NMIgen_CTRL_CMPMODE2_MASK;
                #endif /* (0 != NMIgen_CTRL_CMPMODE2_SHIFT) */

                /*Clear existing mode */
                NMIgen_CONTROL &= ((uint8)(~(NMIgen_CTRL_CMPMODE1_MASK |
                                            NMIgen_CTRL_CMPMODE2_MASK)));
                NMIgen_CONTROL |= (comparemode1masked | comparemode2masked);

            #else
                uint8 temp = comparemode;
            #endif /* (NMIgen_UsingFixedFunction) */
        }
    #endif /* NMIgen_CompareMode1SW */

#else /* UseOneCompareMode */

    #if (NMIgen_CompareMode1SW)


        /*******************************************************************************
        * Function Name: NMIgen_SetCompareMode1
        ********************************************************************************
        *
        * Summary:
        *  This function writes the Compare Mode for the pwm or pwm1 output
        *
        * Parameters:
        *  comparemode:  The new compare mode for the PWM output. Use the compare types
        *                defined in the H file as input arguments.
        *
        * Return:
        *  None
        *
        *******************************************************************************/
        void NMIgen_SetCompareMode1(uint8 comparemode) 
        {
            #if(0 != NMIgen_CTRL_CMPMODE1_SHIFT)
                uint8 comparemodemasked = ((uint8)((uint8)comparemode << NMIgen_CTRL_CMPMODE1_SHIFT)) &
                                           NMIgen_CTRL_CMPMODE1_MASK;
            #else
                uint8 comparemodemasked = comparemode & NMIgen_CTRL_CMPMODE1_MASK;
            #endif /* (0 != NMIgen_CTRL_CMPMODE1_SHIFT) */

            #if (NMIgen_UseControl)
                NMIgen_CONTROL &= ((uint8)(~NMIgen_CTRL_CMPMODE1_MASK)); /*Clear existing mode */
                NMIgen_CONTROL |= comparemodemasked;
            #endif /* (NMIgen_UseControl) */
        }
    #endif /* NMIgen_CompareMode1SW */

#if (NMIgen_CompareMode2SW)


    /*******************************************************************************
    * Function Name: NMIgen_SetCompareMode2
    ********************************************************************************
    *
    * Summary:
    *  This function writes the Compare Mode for the pwm or pwm2 output
    *
    * Parameters:
    *  comparemode:  The new compare mode for the PWM output. Use the compare types
    *                defined in the H file as input arguments.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void NMIgen_SetCompareMode2(uint8 comparemode) 
    {

        #if(0 != NMIgen_CTRL_CMPMODE2_SHIFT)
            uint8 comparemodemasked = ((uint8)((uint8)comparemode << NMIgen_CTRL_CMPMODE2_SHIFT)) &
                                                 NMIgen_CTRL_CMPMODE2_MASK;
        #else
            uint8 comparemodemasked = comparemode & NMIgen_CTRL_CMPMODE2_MASK;
        #endif /* (0 != NMIgen_CTRL_CMPMODE2_SHIFT) */

        #if (NMIgen_UseControl)
            NMIgen_CONTROL &= ((uint8)(~NMIgen_CTRL_CMPMODE2_MASK)); /*Clear existing mode */
            NMIgen_CONTROL |= comparemodemasked;
        #endif /* (NMIgen_UseControl) */
    }
    #endif /*NMIgen_CompareMode2SW */

#endif /* UseOneCompareMode */


#if (!NMIgen_UsingFixedFunction)


    /*******************************************************************************
    * Function Name: NMIgen_WriteCounter
    ********************************************************************************
    *
    * Summary:
    *  Writes a new counter value directly to the counter register. This will be
    *  implemented for that currently running period and only that period. This API
    *  is valid only for UDB implementation and not available for fixed function
    *  PWM implementation.
    *
    * Parameters:
    *  counter:  The period new period counter value.
    *
    * Return:
    *  None
    *
    * Side Effects:
    *  The PWM Period will be reloaded when a counter value will be a zero
    *
    *******************************************************************************/
    void NMIgen_WriteCounter(uint8 counter) \
                                       
    {
        CY_SET_REG8(NMIgen_COUNTER_LSB_PTR, counter);
    }


    /*******************************************************************************
    * Function Name: NMIgen_ReadCounter
    ********************************************************************************
    *
    * Summary:
    *  This function returns the current value of the counter.  It doesn't matter
    *  if the counter is enabled or running.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  The current value of the counter.
    *
    *******************************************************************************/
    uint8 NMIgen_ReadCounter(void) 
    {
        /* Force capture by reading Accumulator */
        /* Must first do a software capture to be able to read the counter */
        /* It is up to the user code to make sure there isn't already captured data in the FIFO */
          (void)CY_GET_REG8(NMIgen_COUNTERCAP_LSB_PTR_8BIT);

        /* Read the data from the FIFO */
        return (CY_GET_REG8(NMIgen_CAPTURE_LSB_PTR));
    }

    #if (NMIgen_UseStatus)


        /*******************************************************************************
        * Function Name: NMIgen_ClearFIFO
        ********************************************************************************
        *
        * Summary:
        *  This function clears all capture data from the capture FIFO
        *
        * Parameters:
        *  None
        *
        * Return:
        *  None
        *
        *******************************************************************************/
        void NMIgen_ClearFIFO(void) 
        {
            while(0u != (NMIgen_ReadStatusRegister() & NMIgen_STATUS_FIFONEMPTY))
            {
                (void)NMIgen_ReadCapture();
            }
        }

    #endif /* NMIgen_UseStatus */

#endif /* !NMIgen_UsingFixedFunction */


/*******************************************************************************
* Function Name: NMIgen_WritePeriod
********************************************************************************
*
* Summary:
*  This function is used to change the period of the counter.  The new period
*  will be loaded the next time terminal count is detected.
*
* Parameters:
*  period:  Period value. May be between 1 and (2^Resolution)-1.  A value of 0
*           will result in the counter remaining at zero.
*
* Return:
*  None
*
*******************************************************************************/
void NMIgen_WritePeriod(uint8 period) 
{
    #if(NMIgen_UsingFixedFunction)
        CY_SET_REG16(NMIgen_PERIOD_LSB_PTR, (uint16)period);
    #else
        CY_SET_REG8(NMIgen_PERIOD_LSB_PTR, period);
    #endif /* (NMIgen_UsingFixedFunction) */
}

#if (NMIgen_UseOneCompareMode)


    /*******************************************************************************
    * Function Name: NMIgen_WriteCompare
    ********************************************************************************
    *
    * Summary:
    *  This funtion is used to change the compare1 value when the PWM is in Dither
    *  mode. The compare output will reflect the new value on the next UDB clock.
    *  The compare output will be driven high when the present counter value is
    *  compared to the compare value based on the compare mode defined in
    *  Dither Mode.
    *
    * Parameters:
    *  compare:  New compare value.
    *
    * Return:
    *  None
    *
    * Side Effects:
    *  This function is only available if the PWM mode parameter is set to
    *  Dither Mode, Center Aligned Mode or One Output Mode
    *
    *******************************************************************************/
    void NMIgen_WriteCompare(uint8 compare) \
                                       
    {
        #if(NMIgen_UsingFixedFunction)
            CY_SET_REG16(NMIgen_COMPARE1_LSB_PTR, (uint16)compare);
        #else
            CY_SET_REG8(NMIgen_COMPARE1_LSB_PTR, compare);
        #endif /* (NMIgen_UsingFixedFunction) */

        #if (NMIgen_PWMMode == NMIgen__B_PWM__DITHER)
            #if(NMIgen_UsingFixedFunction)
                CY_SET_REG16(NMIgen_COMPARE2_LSB_PTR, (uint16)(compare + 1u));
            #else
                CY_SET_REG8(NMIgen_COMPARE2_LSB_PTR, (compare + 1u));
            #endif /* (NMIgen_UsingFixedFunction) */
        #endif /* (NMIgen_PWMMode == NMIgen__B_PWM__DITHER) */
    }


#else


    /*******************************************************************************
    * Function Name: NMIgen_WriteCompare1
    ********************************************************************************
    *
    * Summary:
    *  This funtion is used to change the compare1 value.  The compare output will
    *  reflect the new value on the next UDB clock.  The compare output will be
    *  driven high when the present counter value is less than or less than or
    *  equal to the compare register, depending on the mode.
    *
    * Parameters:
    *  compare:  New compare value.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void NMIgen_WriteCompare1(uint8 compare) \
                                        
    {
        #if(NMIgen_UsingFixedFunction)
            CY_SET_REG16(NMIgen_COMPARE1_LSB_PTR, (uint16)compare);
        #else
            CY_SET_REG8(NMIgen_COMPARE1_LSB_PTR, compare);
        #endif /* (NMIgen_UsingFixedFunction) */
    }


    /*******************************************************************************
    * Function Name: NMIgen_WriteCompare2
    ********************************************************************************
    *
    * Summary:
    *  This funtion is used to change the compare value, for compare1 output.
    *  The compare output will reflect the new value on the next UDB clock.
    *  The compare output will be driven high when the present counter value is
    *  less than or less than or equal to the compare register, depending on the
    *  mode.
    *
    * Parameters:
    *  compare:  New compare value.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void NMIgen_WriteCompare2(uint8 compare) \
                                        
    {
        #if(NMIgen_UsingFixedFunction)
            CY_SET_REG16(NMIgen_COMPARE2_LSB_PTR, compare);
        #else
            CY_SET_REG8(NMIgen_COMPARE2_LSB_PTR, compare);
        #endif /* (NMIgen_UsingFixedFunction) */
    }
#endif /* UseOneCompareMode */

#if (NMIgen_DeadBandUsed)


    /*******************************************************************************
    * Function Name: NMIgen_WriteDeadTime
    ********************************************************************************
    *
    * Summary:
    *  This function writes the dead-band counts to the corresponding register
    *
    * Parameters:
    *  deadtime:  Number of counts for dead time
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void NMIgen_WriteDeadTime(uint8 deadtime) 
    {
        /* If using the Dead Band 1-255 mode then just write the register */
        #if(!NMIgen_DeadBand2_4)
            CY_SET_REG8(NMIgen_DEADBAND_COUNT_PTR, deadtime);
        #else
            /* Otherwise the data has to be masked and offset */
            /* Clear existing data */
            NMIgen_DEADBAND_COUNT &= ((uint8)(~NMIgen_DEADBAND_COUNT_MASK));

            /* Set new dead time */
            #if(NMIgen_DEADBAND_COUNT_SHIFT)
                NMIgen_DEADBAND_COUNT |= ((uint8)((uint8)deadtime << NMIgen_DEADBAND_COUNT_SHIFT)) &
                                                    NMIgen_DEADBAND_COUNT_MASK;
            #else
                NMIgen_DEADBAND_COUNT |= deadtime & NMIgen_DEADBAND_COUNT_MASK;
            #endif /* (NMIgen_DEADBAND_COUNT_SHIFT) */

        #endif /* (!NMIgen_DeadBand2_4) */
    }


    /*******************************************************************************
    * Function Name: NMIgen_ReadDeadTime
    ********************************************************************************
    *
    * Summary:
    *  This function reads the dead-band counts from the corresponding register
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Dead Band Counts
    *
    *******************************************************************************/
    uint8 NMIgen_ReadDeadTime(void) 
    {
        /* If using the Dead Band 1-255 mode then just read the register */
        #if(!NMIgen_DeadBand2_4)
            return (CY_GET_REG8(NMIgen_DEADBAND_COUNT_PTR));
        #else

            /* Otherwise the data has to be masked and offset */
            #if(NMIgen_DEADBAND_COUNT_SHIFT)
                return ((uint8)(((uint8)(NMIgen_DEADBAND_COUNT & NMIgen_DEADBAND_COUNT_MASK)) >>
                                                                           NMIgen_DEADBAND_COUNT_SHIFT));
            #else
                return (NMIgen_DEADBAND_COUNT & NMIgen_DEADBAND_COUNT_MASK);
            #endif /* (NMIgen_DEADBAND_COUNT_SHIFT) */
        #endif /* (!NMIgen_DeadBand2_4) */
    }
#endif /* DeadBandUsed */

#if (NMIgen_UseStatus || NMIgen_UsingFixedFunction)


    /*******************************************************************************
    * Function Name: NMIgen_SetInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  This function configures the interrupts mask control of theinterrupt
    *  source status register.
    *
    * Parameters:
    *  uint8 interruptMode: Bit field containing the interrupt sources enabled
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void NMIgen_SetInterruptMode(uint8 interruptMode) 
    {
        CY_SET_REG8(NMIgen_STATUS_MASK_PTR, interruptMode);
    }


    /*******************************************************************************
    * Function Name: NMIgen_ReadStatusRegister
    ********************************************************************************
    *
    * Summary:
    *  This function returns the current state of the status register.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8 : Current status register value. The status register bits are:
    *  [7:6] : Unused(0)
    *  [5]   : Kill event output
    *  [4]   : FIFO not empty
    *  [3]   : FIFO full
    *  [2]   : Terminal count
    *  [1]   : Compare output 2
    *  [0]   : Compare output 1
    *
    *******************************************************************************/
    uint8 NMIgen_ReadStatusRegister(void) 
    {
        return (CY_GET_REG8(NMIgen_STATUS_PTR));
    }

#endif /* (NMIgen_UseStatus || NMIgen_UsingFixedFunction) */


#if (NMIgen_UseControl)


    /*******************************************************************************
    * Function Name: NMIgen_ReadControlRegister
    ********************************************************************************
    *
    * Summary:
    *  Returns the current state of the control register. This API is available
    *  only if the control register is not removed.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8 : Current control register value
    *
    *******************************************************************************/
    uint8 NMIgen_ReadControlRegister(void) 
    {
        uint8 result;

        result = CY_GET_REG8(NMIgen_CONTROL_PTR);
        return (result);
    }


    /*******************************************************************************
    * Function Name: NMIgen_WriteControlRegister
    ********************************************************************************
    *
    * Summary:
    *  Sets the bit field of the control register. This API is available only if
    *  the control register is not removed.
    *
    * Parameters:
    *  uint8 control: Control register bit field, The status register bits are:
    *  [7]   : PWM Enable
    *  [6]   : Reset
    *  [5:3] : Compare Mode2
    *  [2:0] : Compare Mode2
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void NMIgen_WriteControlRegister(uint8 control) 
    {
        CY_SET_REG8(NMIgen_CONTROL_PTR, control);
    }

#endif /* (NMIgen_UseControl) */


#if (!NMIgen_UsingFixedFunction)


    /*******************************************************************************
    * Function Name: NMIgen_ReadCapture
    ********************************************************************************
    *
    * Summary:
    *  Reads the capture value from the capture FIFO.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8/uint16: The current capture value
    *
    *******************************************************************************/
    uint8 NMIgen_ReadCapture(void) 
    {
        return (CY_GET_REG8(NMIgen_CAPTURE_LSB_PTR));
    }

#endif /* (!NMIgen_UsingFixedFunction) */


#if (NMIgen_UseOneCompareMode)


    /*******************************************************************************
    * Function Name: NMIgen_ReadCompare
    ********************************************************************************
    *
    * Summary:
    *  Reads the compare value for the compare output when the PWM Mode parameter is
    *  set to Dither mode, Center Aligned mode, or One Output mode.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8/uint16: Current compare value
    *
    *******************************************************************************/
    uint8 NMIgen_ReadCompare(void) 
    {
        #if(NMIgen_UsingFixedFunction)
            return ((uint8)CY_GET_REG16(NMIgen_COMPARE1_LSB_PTR));
        #else
            return (CY_GET_REG8(NMIgen_COMPARE1_LSB_PTR));
        #endif /* (NMIgen_UsingFixedFunction) */
    }

#else


    /*******************************************************************************
    * Function Name: NMIgen_ReadCompare1
    ********************************************************************************
    *
    * Summary:
    *  Reads the compare value for the compare1 output.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8/uint16: Current compare value.
    *
    *******************************************************************************/
    uint8 NMIgen_ReadCompare1(void) 
    {
        return (CY_GET_REG8(NMIgen_COMPARE1_LSB_PTR));
    }


    /*******************************************************************************
    * Function Name: NMIgen_ReadCompare2
    ********************************************************************************
    *
    * Summary:
    *  Reads the compare value for the compare2 output.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8/uint16: Current compare value.
    *
    *******************************************************************************/
    uint8 NMIgen_ReadCompare2(void) 
    {
        return (CY_GET_REG8(NMIgen_COMPARE2_LSB_PTR));
    }

#endif /* (NMIgen_UseOneCompareMode) */


/*******************************************************************************
* Function Name: NMIgen_ReadPeriod
********************************************************************************
*
* Summary:
*  Reads the period value used by the PWM hardware.
*
* Parameters:
*  None
*
* Return:
*  uint8/16: Period value
*
*******************************************************************************/
uint8 NMIgen_ReadPeriod(void) 
{
    #if(NMIgen_UsingFixedFunction)
        return ((uint8)CY_GET_REG16(NMIgen_PERIOD_LSB_PTR));
    #else
        return (CY_GET_REG8(NMIgen_PERIOD_LSB_PTR));
    #endif /* (NMIgen_UsingFixedFunction) */
}

#if ( NMIgen_KillModeMinTime)


    /*******************************************************************************
    * Function Name: NMIgen_WriteKillTime
    ********************************************************************************
    *
    * Summary:
    *  Writes the kill time value used by the hardware when the Kill Mode
    *  is set to Minimum Time.
    *
    * Parameters:
    *  uint8: Minimum Time kill counts
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void NMIgen_WriteKillTime(uint8 killtime) 
    {
        CY_SET_REG8(NMIgen_KILLMODEMINTIME_PTR, killtime);
    }


    /*******************************************************************************
    * Function Name: NMIgen_ReadKillTime
    ********************************************************************************
    *
    * Summary:
    *  Reads the kill time value used by the hardware when the Kill Mode is set
    *  to Minimum Time.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  uint8: The current Minimum Time kill counts
    *
    *******************************************************************************/
    uint8 NMIgen_ReadKillTime(void) 
    {
        return (CY_GET_REG8(NMIgen_KILLMODEMINTIME_PTR));
    }

#endif /* ( NMIgen_KillModeMinTime) */

/* [] END OF FILE */
