/*******************************************************************************
* File Name: NMIgen_PM.c
* Version 3.30
*
* Description:
*  This file provides the power management source code to API for the
*  PWM.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "NMIgen.h"

static NMIgen_backupStruct NMIgen_backup;


/*******************************************************************************
* Function Name: NMIgen_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the current user configuration of the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  NMIgen_backup:  Variables of this global structure are modified to
*  store the values of non retention configuration registers when Sleep() API is
*  called.
*
*******************************************************************************/
void NMIgen_SaveConfig(void) 
{

    #if(!NMIgen_UsingFixedFunction)
        #if(!NMIgen_PWMModeIsCenterAligned)
            NMIgen_backup.PWMPeriod = NMIgen_ReadPeriod();
        #endif /* (!NMIgen_PWMModeIsCenterAligned) */
        NMIgen_backup.PWMUdb = NMIgen_ReadCounter();
        #if (NMIgen_UseStatus)
            NMIgen_backup.InterruptMaskValue = NMIgen_STATUS_MASK;
        #endif /* (NMIgen_UseStatus) */

        #if(NMIgen_DeadBandMode == NMIgen__B_PWM__DBM_256_CLOCKS || \
            NMIgen_DeadBandMode == NMIgen__B_PWM__DBM_2_4_CLOCKS)
            NMIgen_backup.PWMdeadBandValue = NMIgen_ReadDeadTime();
        #endif /*  deadband count is either 2-4 clocks or 256 clocks */

        #if(NMIgen_KillModeMinTime)
             NMIgen_backup.PWMKillCounterPeriod = NMIgen_ReadKillTime();
        #endif /* (NMIgen_KillModeMinTime) */

        #if(NMIgen_UseControl)
            NMIgen_backup.PWMControlRegister = NMIgen_ReadControlRegister();
        #endif /* (NMIgen_UseControl) */
    #endif  /* (!NMIgen_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: NMIgen_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration of the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  NMIgen_backup:  Variables of this global structure are used to
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void NMIgen_RestoreConfig(void) 
{
        #if(!NMIgen_UsingFixedFunction)
            #if(!NMIgen_PWMModeIsCenterAligned)
                NMIgen_WritePeriod(NMIgen_backup.PWMPeriod);
            #endif /* (!NMIgen_PWMModeIsCenterAligned) */

            NMIgen_WriteCounter(NMIgen_backup.PWMUdb);

            #if (NMIgen_UseStatus)
                NMIgen_STATUS_MASK = NMIgen_backup.InterruptMaskValue;
            #endif /* (NMIgen_UseStatus) */

            #if(NMIgen_DeadBandMode == NMIgen__B_PWM__DBM_256_CLOCKS || \
                NMIgen_DeadBandMode == NMIgen__B_PWM__DBM_2_4_CLOCKS)
                NMIgen_WriteDeadTime(NMIgen_backup.PWMdeadBandValue);
            #endif /* deadband count is either 2-4 clocks or 256 clocks */

            #if(NMIgen_KillModeMinTime)
                NMIgen_WriteKillTime(NMIgen_backup.PWMKillCounterPeriod);
            #endif /* (NMIgen_KillModeMinTime) */

            #if(NMIgen_UseControl)
                NMIgen_WriteControlRegister(NMIgen_backup.PWMControlRegister);
            #endif /* (NMIgen_UseControl) */
        #endif  /* (!NMIgen_UsingFixedFunction) */
    }


/*******************************************************************************
* Function Name: NMIgen_Sleep
********************************************************************************
*
* Summary:
*  Disables block's operation and saves the user configuration. Should be called
*  just prior to entering sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  NMIgen_backup.PWMEnableState:  Is modified depending on the enable
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void NMIgen_Sleep(void) 
{
    #if(NMIgen_UseControl)
        if(NMIgen_CTRL_ENABLE == (NMIgen_CONTROL & NMIgen_CTRL_ENABLE))
        {
            /*Component is enabled */
            NMIgen_backup.PWMEnableState = 1u;
        }
        else
        {
            /* Component is disabled */
            NMIgen_backup.PWMEnableState = 0u;
        }
    #endif /* (NMIgen_UseControl) */

    /* Stop component */
    NMIgen_Stop();

    /* Save registers configuration */
    NMIgen_SaveConfig();
}


/*******************************************************************************
* Function Name: NMIgen_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration. Should be called just after
*  awaking from sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  NMIgen_backup.pwmEnable:  Is used to restore the enable state of
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void NMIgen_Wakeup(void) 
{
     /* Restore registers values */
    NMIgen_RestoreConfig();

    if(NMIgen_backup.PWMEnableState != 0u)
    {
        /* Enable component's operation */
        NMIgen_Enable();
    } /* Do nothing if component's block was disabled before */

}


/* [] END OF FILE */
