#include <project.h>

// Every call to read() returns a single byte for each
// keystroke.  These configure what byte will be returned
// for each "special" key.  To ignore a key, use zero.
#define PS2_TAB				9
#define PS2_ENTER			13
#define PS2_LINEFEED		10
#define PS2_BACKSPACE		127
#define PS2_ESC				27
#define PS2_INSERT			0
#define PS2_DELETE			127
#define PS2_HOME			0
#define PS2_END				0
#define PS2_PAGEUP			25
#define PS2_PAGEDOWN		26
#define PS2_UPARROW			11
#define PS2_LEFTARROW		8
#define PS2_DOWNARROW		12
#define PS2_RIGHTARROW		21
#define PS2_F1				0
#define PS2_F2				0
#define PS2_F3				0
#define PS2_F4				0
#define PS2_F5				0
#define PS2_F6				0
#define PS2_F7				0
#define PS2_F8				0
#define PS2_F9				0
#define PS2_F10				0
#define PS2_F11				0
#define PS2_F12				0
#define PS2_SCROLL			0

#define PS2_KEYMAP_SIZE 136

typedef struct {
	uint8 noshift[PS2_KEYMAP_SIZE];
	uint8 shift[PS2_KEYMAP_SIZE];
	uint8 uses_altgr;
	uint8 altgr[PS2_KEYMAP_SIZE];
} PS2Keymap_t;

static uint8 CharBuffer=0;
static uint8 UTF8next=0;

#define BUFFER_SIZE 16
uint8 ps2buf[BUFFER_SIZE];
uint8 ps2head,ps2tail;

const PS2Keymap_t keymap = {
  // without shift
	{0, PS2_F9, 0, PS2_F5, PS2_F3, PS2_F1, PS2_F2, PS2_F12,
	0, PS2_F10, PS2_F8, PS2_F6, PS2_F4, PS2_TAB, '`', 0,
	0, 0 /*Lalt*/, 0 /*Lshift*/, 0, 0 /*Lctrl*/, 'q', '1', 0,
	0, 0, 'z', 's', 'a', 'w', '2', 0,
	0, 'c', 'x', 'd', 'e', '4', '3', 0,
	0, ' ', 'v', 'f', 't', 'r', '5', 0,
	0, 'n', 'b', 'h', 'g', 'y', '6', 0,
	0, 0, 'm', 'j', 'u', '7', '8', 0,
	0, ',', 'k', 'i', 'o', '0', '9', 0,
	0, '.', '/', 'l', ';', 'p', '-', 0,
	0, 0, '\'', 0, '[', '=', 0, 0,
	0 /*CapsLock*/, 0 /*Rshift*/, PS2_ENTER /*Enter*/, ']', 0, '\\', 0, 0,
	0, 0, 0, 0, 0, 0, PS2_BACKSPACE, 0,
	0, '1', 0, '4', '7', 0, 0, 0,
	'0', '.', '2', '5', '6', '8', PS2_ESC, 0 /*NumLock*/,
	PS2_F11, '+', '3', '-', '*', '9', PS2_SCROLL, 0,
	0, 0, 0, PS2_F7 },
  // with shift
	{0, PS2_F9, 0, PS2_F5, PS2_F3, PS2_F1, PS2_F2, PS2_F12,
	0, PS2_F10, PS2_F8, PS2_F6, PS2_F4, PS2_TAB, '~', 0,
	0, 0 /*Lalt*/, 0 /*Lshift*/, 0, 0 /*Lctrl*/, 'Q', '!', 0,
	0, 0, 'Z', 'S', 'A', 'W', '@', 0,
	0, 'C', 'X', 'D', 'E', '$', '#', 0,
	0, ' ', 'V', 'F', 'T', 'R', '%', 0,
	0, 'N', 'B', 'H', 'G', 'Y', '^', 0,
	0, 0, 'M', 'J', 'U', '&', '*', 0,
	0, '<', 'K', 'I', 'O', ')', '(', 0,
	0, '>', '?', 'L', ':', 'P', '_', 0,
	0, 0, '"', 0, '{', '+', 0, 0,
	0 /*CapsLock*/, 0 /*Rshift*/, PS2_ENTER /*Enter*/, '}', 0, '|', 0, 0,
	0, 0, 0, 0, 0, 0, PS2_BACKSPACE, 0,
	0, '1', 0, '4', '7', 0, 0, 0,
	'0', '.', '2', '5', '6', '8', PS2_ESC, 0 /*NumLock*/,
	PS2_F11, '+', '3', '-', '*', '9', PS2_SCROLL, 0,
	0, 0, 0, PS2_F7 },
	0,{}
};

#define BREAK     0x01
#define MODIFIER  0x02
#define SHIFT_L   0x04
#define SHIFT_R   0x08
#define ALTGR     0x10
#define CTRL      0x20

#define PS2_INTERRUPT 1

#if PS2_INTERRUPT
static inline uint8_t get_scan_code(void)
{
	uint8_t c, i;

	i = ps2tail;
	if (i == ps2head) return 0;
	i++;
	if (i >= BUFFER_SIZE) i = 0;
	c = ps2buf[i];
	ps2tail = i;
	return c;
}
#else
static inline uint8_t get_scan_code(void)  
{
    if ((PS2_1_StatusReg_1_Read()&0x8)==0) return 0;
    return PS2_1_Datapath_1_F0_REG;
}    
#endif

static char get_iso8859_code(void)
{
	static uint8 state=0;
	uint8 s;
	char c;

	while (1) {
		s = get_scan_code();
		if (!s) return 0;
		if (s == 0xF0) {
			state |= BREAK;
		} else if (s == 0xE0) {
			state |= MODIFIER;
		} else {
			if (state & BREAK) {
				if (s == 0x12) {
					state &= ~SHIFT_L;
				} else if (s == 0x59) {
					state &= ~SHIFT_R;
				} else if (s == 0x14) {
					state &= ~CTRL;
				} else if (s == 0x11 && (state & MODIFIER)) {
					state &= ~ALTGR;
				}
				// CTRL, ALT & WIN keys could be added
				// but is that really worth the overhead?
				state &= ~(BREAK | MODIFIER);
				continue;
			}
			if (s == 0x12) {
				state |= SHIFT_L;
				continue;
			} else if (s == 0x59) {
				state |= SHIFT_R;
				continue;
			} else if (s == 0x14) {
				state |= CTRL;
				continue;
			} else if (s == 0x11 && (state & MODIFIER)) {
				state |= ALTGR;
			}
			c = 0;
			if (state & MODIFIER) {
				switch (s) {
				  case 0x70: c = PS2_INSERT;      break;
				  case 0x6C: c = PS2_HOME;        break;
				  case 0x7D: c = PS2_PAGEUP;      break;
				  case 0x71: c = PS2_DELETE;      break;
				  case 0x69: c = PS2_END;         break;
				  case 0x7A: c = PS2_PAGEDOWN;    break;
				  case 0x75: c = PS2_UPARROW;     break;
				  case 0x6B: c = PS2_LEFTARROW;   break;
				  case 0x72: c = PS2_DOWNARROW;   break;
				  case 0x74: c = PS2_RIGHTARROW;  break;
				  case 0x4A: c = '/';             break;
				  case 0x5A: c = PS2_ENTER;       break;
				  default: break;
				}
			} else if ((state & ALTGR) && keymap.uses_altgr) {
				if (s < PS2_KEYMAP_SIZE)
					c = keymap.altgr[s];
			} else if (state & (SHIFT_L | SHIFT_R)) {
				if (s < PS2_KEYMAP_SIZE)
					c = keymap.shift[s];
			} else {
				if (s < PS2_KEYMAP_SIZE)
					c = keymap.noshift[s];
			}
			if (state & CTRL) {
				if (c >= 'A' && c <= 'Z')
					c = c - 'A' + 1;
				else if (c >= 'a' && c <= 'z')
					c = c - 'a' + 1;
				else if (c == PS2_ENTER)
					c = PS2_LINEFEED;
			}
			state &= ~(BREAK | MODIFIER);
			if (c) return c;
		}
	}
}

#if PS2_INTERRUPT
CY_ISR(ps2interrupt)
{
 uint8 i;
 while (PS2_1_StatusReg_1_Read()&0x08)      // FIFO Not Empty
  {
    i=ps2head+1; if (i>=BUFFER_SIZE) {i=0;}
    if (i!=ps2tail) {ps2buf[i]=PS2_1_Datapath_1_F0_REG;ps2head=i;}
  }
}
#endif

void ps2_init(void)
{
#if PS2_INTERRUPT
    ps2head=ps2tail=0;
    KeyInt_StartEx(ps2interrupt);
#endif    
}

uint8 ps2_available(void) {
	if (CharBuffer || UTF8next) return 1;
	CharBuffer = get_iso8859_code();
	if (CharBuffer) return 1;
	return 0;
}

uint8 ps2_read(void) {
	uint8_t result;

	result = UTF8next;
	if (result) {
		UTF8next = 0;
	} else {
		result = CharBuffer;
		if (result) {
			CharBuffer = 0;
		} else {
			result = get_iso8859_code();
		}
		if (result >= 128) {
			UTF8next = (result & 0x3F) | 0x80;
			result = ((result >> 6) & 0x1F) | 0xC0;
		}
	}
	if (!result) return -1;
	return result;
}
